@@ -21,6 +21,7 @@ import openmdao.api as om
 from openmdao.components.exec_comp import _expr_dict, _temporary_expr_dict
 from openmdao.utils.assert_utils import assert_near_equal, assert_check_partials, assert_warning
 from openmdao.utils.general_utils import env_truthy
+from openmdao.utils.testing_utils import force_check_partials
 from openmdao.utils.om_warnings import OMDeprecationWarning, SetupWarning
 
 _ufunc_test_data = {
@@ -278,28 +279,6 @@ _ufunc_test_data = {
 }
 
 
-# 'factorial' will raise a RuntimeError or a deprecation warning depending on scipy version
-if Version(scipy.__version__) >= Version("1.5.0"):
-    _ufunc_test_data['factorial'] = {
-        'str': 'f=factorial(x)',
-        'args': {'f': {'val': np.zeros(6)},
-                 'x': {'val': np.random.random(6)}},
-        'error': (RuntimeError,
-                  "The 'factorial' function is not supported for SciPy "
-                  f"versions >= 1.5, current version: {scipy.__version__}")
-    }
-else:
-    _ufunc_test_data['factorial'] = {
-        'str': 'f=factorial(x)',
-        'check_func': scipy.special.factorial,
-        'args': {'f': {'val': np.zeros(6)},
-                 'x': {'val': np.random.random(6)}},
-        'warning': (OMDeprecationWarning,
-                    "The 'factorial' function is deprecated. "
-                    "It is no longer supported for SciPy versions >= 1.5.")
-    }
-
-
 class TestExecComp(unittest.TestCase):
 
     def test_missing_partial_warn(self):
@@ -706,7 +685,7 @@ class TestExecComp(unittest.TestCase):
         prob.set_solver_print(level=0)
         prob.run_model()
 
-        data = prob.check_partials(out_stream=None)
+        data = force_check_partials(prob, out_stream=None)
 
         assert_check_partials(data, atol=1e-5, rtol=1e-5)
 
@@ -720,7 +699,7 @@ class TestExecComp(unittest.TestCase):
         prob.set_solver_print(level=0)
         prob.run_model()
 
-        data = prob.check_partials(out_stream=None)
+        data = force_check_partials(prob, out_stream=None)
 
         assert_check_partials(data, atol=1e-5, rtol=1e-5)
 
@@ -854,9 +833,7 @@ class TestExecComp(unittest.TestCase):
         model.add_subsystem('comp', comp)
         p.setup()
 
-        declared_partials = comp._declared_partials[('y','x')]
-        self.assertTrue('rows' not in declared_partials )
-        self.assertTrue('cols' not in declared_partials )
+        self.assertEqual(len(comp._declared_partials_patterns), 0)
 
         # run with has_diag_partials=True
         p = om.Problem()
@@ -866,11 +843,11 @@ class TestExecComp(unittest.TestCase):
         p.setup()
         p.final_setup()
 
-        declared_partials = comp._declared_partials[('y','x')]
+        declared_partials = comp._declared_partials_patterns[('y','x')]
         self.assertTrue('rows' in declared_partials )
-        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials[('y','x')]['rows']))
+        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials_patterns[('y','x')]['rows']))
         self.assertTrue('cols' in declared_partials )
-        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials[('y','x')]['cols']))
+        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials_patterns[('y','x')]['cols']))
 
     def test_exec_comp_deriv_sparsity(self):
         # Check to make sure that when an ExecComp has more than one
@@ -885,7 +862,7 @@ class TestExecComp(unittest.TestCase):
         p.final_setup()
 
         ## make sure only the partials that are needed are declared
-        declared_partials = comp._declared_partials
+        declared_partials = comp._declared_partials_patterns
         self.assertListEqual( sorted([('y1', 'x1'), ('y2', 'x2') ]),
                               sorted(declared_partials.keys()))
 
@@ -912,7 +889,7 @@ class TestExecComp(unittest.TestCase):
         p.setup()
         p.final_setup()
 
-        declared_partials = comp._declared_partials
+        declared_partials = comp._declared_partials_patterns
         self.assertListEqual( sorted([('y1', 'x1'), ('y2', 'x2') ]),
                               sorted(declared_partials.keys()))
 
@@ -931,17 +908,17 @@ class TestExecComp(unittest.TestCase):
         p.setup()
         p.final_setup()
 
-        declared_partials = comp._declared_partials
+        declared_partials = comp._declared_partials_patterns
         self.assertListEqual( sorted([('y1', 'x1'), ('y2', 'x2') ]),
                               sorted(declared_partials.keys()))
         self.assertTrue('cols' in declared_partials[('y1', 'x1')] )
         self.assertTrue('rows' in declared_partials[('y1', 'x1')] )
         self.assertTrue('cols' in declared_partials[('y2', 'x2')] )
         self.assertTrue('rows' in declared_partials[('y2', 'x2')] )
-        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials[('y1','x1')]['rows']))
-        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials[('y1','x1')]['cols']))
-        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials[('y2','x2')]['rows']))
-        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials[('y2','x2')]['cols']))
+        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials_patterns[('y1','x1')]['rows']))
+        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials_patterns[('y1','x1')]['cols']))
+        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials_patterns[('y2','x2')]['rows']))
+        self.assertListEqual([0,1,2,3,4], list( comp._declared_partials_patterns[('y2','x2')]['cols']))
 
         p.run_model()
 
@@ -967,10 +944,9 @@ class TestExecComp(unittest.TestCase):
     def test_tags(self):
         prob = om.Problem(model=om.Group())
         C1 = prob.model.add_subsystem('C1', om.ExecComp('y=x+z+1.',
-                                                     x={'val': 1.0, 'units': 'm', 'tags': 'tagx'},
-                                                     y={'units': 'm', 'tags': ['tagy','tagq']},
-                                                     z={'val': 2.0, 'tags': 'tagz'},
-                                                     ))
+                                                        x={'val': 1.0, 'units': 'm', 'tags': 'tagx'},
+                                                        y={'units': 'm', 'tags': ['tagy','tagq']},
+                                                        z={'val': 2.0, 'tags': 'tagz'}))
 
         prob.setup(check=False)
 
@@ -978,49 +954,49 @@ class TestExecComp(unittest.TestCase):
         prob.run_model()
 
         # Inputs no tags
-        inputs = prob.model.list_inputs(values=False, out_stream=None)
+        inputs = prob.model.list_inputs(val=False, prom_name=False, out_stream=None)
         self.assertEqual(sorted(inputs), [
             ('C1.x', {}),
             ('C1.z', {}),
         ])
 
         # Inputs with tags
-        inputs = prob.model.list_inputs(values=False, out_stream=None, tags="tagx")
+        inputs = prob.model.list_inputs(val=False, prom_name=False, out_stream=None, tags="tagx")
         self.assertEqual(sorted(inputs), [
             ('C1.x', {}),
         ])
 
         # Inputs with multiple tags
-        inputs = prob.model.list_inputs(values=False, out_stream=None, tags=["tagx", "tagz"])
+        inputs = prob.model.list_inputs(val=False, prom_name=False, out_stream=None, tags=["tagx", "tagz"])
         self.assertEqual(sorted(inputs), [
             ('C1.x', {}),
             ('C1.z', {}),
         ])
 
         # Inputs with tag that does not match
-        inputs = prob.model.list_inputs(values=False, out_stream=None, tags="tag_wrong")
+        inputs = prob.model.list_inputs(val=False, prom_name=False, out_stream=None, tags="tag_wrong")
         self.assertEqual(sorted(inputs), [])
 
         # Outputs no tags
-        outputs = prob.model.list_outputs(values=False, out_stream=None)
+        outputs = prob.model.list_outputs(val=False, prom_name=False, out_stream=None)
         self.assertEqual(sorted(outputs), [
             ('C1.y', {}),
         ])
 
         # Outputs with tags
-        outputs = prob.model.list_outputs(values=False, out_stream=None, tags="tagy")
+        outputs = prob.model.list_outputs(val=False, prom_name=False, out_stream=None, tags="tagy")
         self.assertEqual(sorted(outputs), [
             ('C1.y', {}),
         ])
 
         # Outputs with multiple tags
-        outputs = prob.model.list_outputs(values=False, out_stream=None, tags=["tagy", "tagx"])
+        outputs = prob.model.list_outputs(val=False, prom_name=False, out_stream=None, tags=["tagy", "tagx"])
         self.assertEqual(sorted(outputs), [
             ('C1.y', {}),
         ])
 
         # Outputs with tag that does not match
-        outputs = prob.model.list_outputs(values=False, out_stream=None, tags="tag_wrong")
+        outputs = prob.model.list_outputs(val=False, prom_name=False, out_stream=None, tags="tag_wrong")
         self.assertEqual(sorted(outputs), [])
 
     def test_feature_has_diag_partials(self):
@@ -1169,9 +1145,9 @@ class TestExecComp(unittest.TestCase):
             "quad_1",
             om.ExecComp(
                 "y = a * x ** 2 + b * x + c",
-                a={"value": 2.0},
-                b={"value": 5.0},
-                c={"value": 3.0},
+                a={"val": 2.0},
+                b={"val": 5.0},
+                c={"val": 3.0},
                 x={"shape": (2,)},
                 y={"shape": (2,)},
             ),
@@ -1348,48 +1324,23 @@ class TestExecComp(unittest.TestCase):
                             x={'val' : 3.0, 'units' : 'cm'},
                             z={'shape' : (1, ), 'units' : 's'})
 
-        self.assertEquals(cm.exception.args[0],
-                          "Defaults for 'x' have already been defined in a previous "
-                          "expression.")
+        self.assertEqual(cm.exception.args[0],
+                         "Defaults for 'x' have already been defined in a previous "
+                         "expression.")
 
         with self.assertRaises(TypeError) as cm:
             excomp.add_expr(p)
 
-        self.assertEquals(cm.exception.args[0],
-                          "Argument 'expr' must be of type 'str', but type 'Problem' was found.")
+        self.assertEqual(cm.exception.args[0],
+                         "Argument 'expr' must be of type 'str', but type 'Problem' was found.")
 
         excomp.add_expr('y = 2.9*x')
         p.model.add_subsystem('zzz', excomp)
         with self.assertRaises(RuntimeError) as cm:
             p.setup()
 
-        self.assertEquals(cm.exception.args[0],
-                          "'zzz' <class ExecComp>: The output 'y' has already been defined by an expression.")
-
-    def test_value_deprecation(self):
-        p = om.Problem()
-
-        msg = ("'zzz' <class ExecComp>: 'value' will be deprecated in 4.0. Please use 'val' in the future.")
-
-        excomp = om.ExecComp('y=x**2', x={'value': np.ones(10)}, y={'val': np.ones(10)})
-
-        p.model.add_subsystem('zzz', excomp)
-        with assert_warning(OMDeprecationWarning, msg):
-            p.setup()
-
-    def test_val_value_error(self):
-        p = om.Problem()
-
-        msg = ("Cannot use 'val' and 'value' at the same time, use 'val'.")
-
-        excomp = om.ExecComp('y=x**2', x={'value': np.ones(10), 'val': np.ones(10)},
-                                       y={'val': np.ones(10)})
-
-        p.model.add_subsystem('zzz', excomp)
-
-        with self.assertRaises(RuntimeError) as cm:
-            p.setup()
-            self.assertEquals(cm.exception.args[0], msg)
+        self.assertEqual(cm.exception.args[0],
+                         "'zzz' <class ExecComp>: The output 'y' has already been defined by an expression.")
 
     def test_feature_add_expr(self):
 
@@ -1517,9 +1468,8 @@ class TestExecComp(unittest.TestCase):
 
         comp = model.add_subsystem('comp', om.ExecComp('y=x+a',
                                                        y={'shape': (3,), 'val': np.zeros(3)},
-
-                                                x=np.array([1., 2., 3.]),
-                                                a={ 'val': np.array([1., 2., 3.]), 'constant':True}))
+                                                       x=np.array([1., 2., 3.]),
+                                                       a={ 'val': np.array([1., 2., 3.]), 'constant':True}))
 
         prob.setup()
 
@@ -1600,7 +1550,7 @@ class TestFunctionRegistration(unittest.TestCase):
             p.run_model()
             assert_near_equal(p['comp.area_square'], np.ones(size) * 9., 1e-6)
 
-            data = p.check_partials(out_stream=None, step=1e-7)
+            data = force_check_partials(p, out_stream=None, step=1e-7)
             self.assertEqual(list(data), ['comp'])
 
     def test_register_check_partials_not_safe_err(self):
@@ -1616,9 +1566,9 @@ class TestFunctionRegistration(unittest.TestCase):
             assert_near_equal(p['comp.area_square'], np.ones(size) * 9., 1e-6)
 
             with self.assertRaises(Exception) as cm:
-                data = p.check_partials(out_stream=None)
-            self.assertEquals(cm.exception.args[0],
-                              "'comp' <class ExecComp>: expression contains functions ['area'] that are not complex safe. To fix this, call declare_partials('*', ['x'], method='fd') on this component prior to setup.")
+                data = force_check_partials(p, out_stream=None)
+            self.assertEqual(cm.exception.args[0],
+                             "'comp' <class ExecComp>: expression contains functions ['area'] that are not complex safe. To fix this, call declare_partials('*', ['x'], method='fd') on this component prior to setup.")
 
     def test_register_check_partials_not_safe_mult_expr(self):
         with _temporary_expr_dict():
@@ -1649,7 +1599,7 @@ class TestFunctionRegistration(unittest.TestCase):
             assert_near_equal(J['comp.out2', 'comp.y'], np.eye(size) * 2. * y, 1e-11)
             assert_near_equal(J['comp.out2', 'comp.z'], np.eye(size), 1e-6)
 
-            data = p.check_partials(out_stream=None, step=1e-7)
+            data = force_check_partials(p, out_stream=None, step=1e-7)
             self.assertEqual(list(data), ['comp'])
 
     def test_register_check_partials_not_safe_mult_expr_err(self):
@@ -1670,9 +1620,9 @@ class TestFunctionRegistration(unittest.TestCase):
             assert_near_equal(p['comp.out2'], np.ones(size) * 21., 1e-6)
 
             with self.assertRaises(Exception) as cm:
-                data = p.check_partials(out_stream=None, step=1e-7)
-            self.assertEquals(cm.exception.args[0],
-                              "'comp' <class ExecComp>: expression contains functions ['unsafe'] that are not complex safe. To fix this, call declare_partials('*', ['z'], method='fd') on this component prior to setup.")
+                data = force_check_partials(p, out_stream=None, step=1e-7)
+            self.assertEqual(cm.exception.args[0],
+                             "'comp' <class ExecComp>: expression contains functions ['unsafe'] that are not complex safe. To fix this, call declare_partials('*', ['z'], method='fd') on this component prior to setup.")
 
     def test_register_check_partials_safe(self):
         with _temporary_expr_dict():
@@ -1685,11 +1635,8 @@ class TestFunctionRegistration(unittest.TestCase):
             p.run_model()
             assert_near_equal(p['comp.area_square'], np.ones(size) * 9., 1e-11)
 
-            data = p.check_partials(out_stream=None)
-            if env_truthy('CI'):
-                self.assertEqual(list(data), ['comp'])
-            else:
-                self.assertEqual(list(data), [])
+            data = force_check_partials(p, out_stream=None)
+            self.assertEqual(list(data), ['comp'])
 
     def test_register_simple_arr_manual_partials_cs(self):
         with _temporary_expr_dict():
@@ -1824,20 +1771,20 @@ class TestFunctionRegistration(unittest.TestCase):
         with _temporary_expr_dict():
             with self.assertRaises(Exception) as cm:
                 om.ExecComp.register('shape', lambda x: x, complex_safe=True)
-            self.assertEquals(cm.exception.args[0], "ExecComp: cannot register name 'shape' because "
-                              "it's a reserved keyword.")
+            self.assertEqual(cm.exception.args[0], "ExecComp: cannot register name 'shape' because "
+                             "it's a reserved keyword.")
 
     def test_register_err_not_callable(self):
         with _temporary_expr_dict():
             with self.assertRaises(Exception) as cm:
                 om.ExecComp.register('foo', 99, complex_safe=True)
-            self.assertEquals(cm.exception.args[0], "ExecComp: 'foo' passed to register() of type 'int' is not callable.")
+            self.assertEqual(cm.exception.args[0], "ExecComp: 'foo' passed to register() of type 'int' is not callable.")
 
     def test_register_err_dup(self):
         with _temporary_expr_dict():
             with self.assertRaises(Exception) as cm:
                 om.ExecComp.register('exp', lambda x: x, complex_safe=True)
-            self.assertEquals(cm.exception.args[0], "ExecComp: 'exp' has already been registered.")
+            self.assertEqual(cm.exception.args[0], "ExecComp: 'exp' has already been registered.")
 
 
 _MASK = np.array(
@@ -1984,7 +1931,7 @@ class TestExecCompParameterized(unittest.TestCase):
 
         if 'check_val' not in test_data:
             try:
-                prob.check_partials(out_stream=None)
+                force_check_partials(prob, out_stream=None)
             except TypeError as e:
                 print(f, 'does not support complex-step differentiation')
 
@@ -2024,7 +1971,7 @@ class TestExecCompParameterized(unittest.TestCase):
             prob.run_model()
 
         if 'check_val' not in test_data:
-            cpd = prob.check_partials(out_stream=None)
+            cpd = force_check_partials(prob, out_stream=None)
 
             for comp in cpd:
                 for (var, wrt) in cpd[comp]:
