@@ -14,10 +14,11 @@ from openmdao.core.driver import Driver
 from openmdao.test_suite.components.paraboloid import Paraboloid
 from openmdao.test_suite.components.misc_components import MultComp
 from openmdao.test_suite.components.sellar import SellarDerivatives, SellarDerivativesConnected
-from openmdao.utils.assert_utils import assert_near_equal, assert_warning
+from openmdao.utils.assert_utils import assert_near_equal, assert_warning, assert_check_totals
 import openmdao.utils.hooks as hooks
 from openmdao.utils.units import convert_units
 from openmdao.utils.om_warnings import DerivativesWarning
+from openmdao.utils.testing_utils import use_tempdirs
 from openmdao.utils.tests.test_hooks import hooks_active
 
 try:
@@ -40,7 +41,8 @@ class TestProblem(unittest.TestCase):
             def compute_partials(self, inputs, J):
                 J['bar', 'foo'] = 1.
 
-        p = om.Problem(model=TestComp())
+        p = om.Problem()
+        p.model.add_subsystem('comp', TestComp(), promotes=['*'])
         p.setup()
 
         p.set_val('foo', 5, units='lbf')
@@ -255,7 +257,7 @@ class TestProblem(unittest.TestCase):
         with self.assertRaisesRegex(ValueError,
                 "<model> <class Group>: Failed to set value of '.*': could not broadcast input array from shape (.*) into shape (.*)."):
             prob.final_setup()
-        prob._initial_condition_cache = {}
+        prob.model._initial_condition_cache = {}
 
         # check assign scalar to array
         arr_val = new_val*np.ones((10, 1))
@@ -363,7 +365,7 @@ class TestProblem(unittest.TestCase):
             p.compute_totals()
 
         self.assertEqual(str(cm.exception),
-                         "Driver is not providing any design variables for compute_totals.")
+                         "No design variables were passed to compute_totals and the driver is not providing any.")
 
     def test_compute_totals_no_args_no_response(self):
         p = om.Problem()
@@ -384,7 +386,7 @@ class TestProblem(unittest.TestCase):
             p.compute_totals()
 
         self.assertEqual(str(cm.exception),
-                         "Driver is not providing any response variables for compute_totals.")
+                         "No response variables were passed to compute_totals and the driver is not providing any.")
 
     def test_compute_totals_no_args(self):
         p = om.Problem()
@@ -422,7 +424,7 @@ class TestProblem(unittest.TestCase):
 
         derivs = p.compute_totals()
 
-        assert_near_equal(derivs['calc.y', 'des_vars.x'], [[2.0]], 1e-6)
+        assert_near_equal(derivs['y', 'x'], [[2.0]], 1e-6)
 
     @parameterized.expand(itertools.product(['fwd', 'rev']))
     def test_compute_jacvec_product(self, mode):
@@ -737,17 +739,23 @@ class TestProblem(unittest.TestCase):
         prob.model.add_subsystem('comp', Paraboloid())
         prob.setup()
         prob.run_model()
-        msg = "Problem .*: To enable complex step, specify 'force_alloc_complex=True' when calling " + \
-            "setup on the problem, e\.g\. 'problem\.setup\(force_alloc_complex=True\)'"
-        with self.assertRaisesRegex(RuntimeError, msg):
+
+        msg = f"Problem {prob._get_inst_id()}: To enable complex step, specify 'force_alloc_complex=True' when calling " + \
+              "setup on the problem, e.g. 'problem.setup(force_alloc_complex=True)'"
+
+        with self.assertRaises(RuntimeError) as err:
             prob.set_complex_step_mode(True)
+        self.assertEqual(str(err.exception), msg)
 
         prob = om.Problem()
         prob.model.add_subsystem('comp', Paraboloid())
-        msg = "Problem .*: set_complex_step_mode cannot be called before `Problem\.run_model\(\)`, " + \
-            "`Problem\.run_driver\(\)`, or `Problem\.final_setup\(\)`."
-        with self.assertRaisesRegex(RuntimeError, msg) :
+
+        msg = f"Problem {prob._get_inst_id()}: set_complex_step_mode cannot be called before `Problem.run_model()`, " + \
+              "`Problem.run_driver()`, or `Problem.final_setup()`."
+
+        with self.assertRaises(RuntimeError) as err:
             prob.set_complex_step_mode(True)
+        self.assertEqual(str(err.exception), msg)
 
     def test_feature_run_driver(self):
 
@@ -990,46 +998,6 @@ class TestProblem(unittest.TestCase):
         # using absolute value will give us the value of the input C2.x, in its units of 'mm'
         assert_near_equal(prob.get_val('G1.C2.x'), 2000.0, 1e-6)
 
-    def test_feature_get_set_with_src_indices_diff(self):
-
-        prob = om.Problem()
-        G1 = prob.model.add_subsystem('G1', om.Group())
-        G1.add_subsystem('C1', om.ExecComp('y=x*2.',
-                                            x={'val': 1.0, 'units': 'cm', 'src_indices': [0], 'flat_src_indices': True},
-                                            y={'val': 0.0, 'units': 'cm'}),
-                         promotes=['x'])
-        G1.add_subsystem('C2', om.ExecComp('y=x*3.',
-                                            x={'val': np.ones(2), 'units': 'mm', 'src_indices': [1,2], 'flat_src_indices': True},
-                                            y={'val': np.zeros(2), 'units': 'mm'}),
-                         promotes=['x'])
-        G1.add_subsystem('C3', om.ExecComp('y=x*4.',
-                                            x={'val': np.ones(3), 'units': 'mm'},
-                                            y={'val': np.zeros(3), 'units': 'mm'}),
-                         promotes=['x'])
-
-        # units and value to use for the _auto_ivc output are ambiguous.  This fixes that.
-        G1.set_input_defaults('x', units='m', val=np.ones(3))
-
-        prob.setup()
-
-        # set G1.x to 2.0 m, based on the units we gave in the set_input_defaults call
-        prob['G1.x'] = np.ones(3) * 2.0
-
-        prob.run_model()
-
-        # we gave 'G1.x' units of 'm' in the set_input_defaults call
-        assert_near_equal(prob['G1.x'], np.ones(3) * 2.0, 1e-6)
-
-        # using absolute value will give us the value of the input C1.x, in its units of 'cm'
-        assert_near_equal(prob['G1.C1.x'], 200.0, 1e-6)
-
-        assert_near_equal(prob['G1.C1.y'], 400.0, 1e-6)
-
-        # using absolute value will give us the value of the input C2.x, in its units of 'mm'
-        assert_near_equal(prob['G1.C2.x'], np.ones(2) * 2000.0, 1e-6)
-
-        assert_near_equal(prob['G1.C2.y'], np.ones(2) * 6000.0, 1e-6)
-
     def test_feature_get_set_with_units_prom_plus_explicit(self):
 
         prob = om.Problem()
@@ -1340,7 +1308,7 @@ class TestProblem(unittest.TestCase):
         self.assertTrue(isinstance(prob.driver, om.ScipyOptimizeDriver))
 
         # invalid model
-        msg = "Problem .*: The value provided for 'model' is not a valid System."
+        msg = "Problem .*: The value provided for 'model' is not a Group."
         with self.assertRaisesRegex(TypeError, msg) :
             prob = om.Problem(om.ScipyOptimizeDriver())
 
@@ -1378,59 +1346,34 @@ class TestProblem(unittest.TestCase):
         model.connect('G2.C5.x', 'C8.b')
         model.connect('G2.C7.x', 'C8.a')
 
+        model.add_design_var('indep1.x')
+        model.add_design_var('indep2.x')
+        model.add_constraint('C8.y')
+        model.add_constraint('Unconnected.y')
+
         p.setup(check=False, mode='rev')
+        p.final_setup()
 
-        relevant = model.get_relevant_vars({'indep1.x': {}, 'indep2.x': {}},
-                                           {'C8.y': {}, 'Unconnected.y': {}}, mode='rev')
+        relevant = model._relevance
 
-        indep1_ins = set(['C3.b', 'C3.c', 'C8.b', 'G1.C1.a', 'G2.C5.a', 'G2.C5.b'])
-        indep1_outs = set(['C3.y', 'C8.y', 'G1.C1.z', 'G2.C5.x', 'indep1.x'])
-        indep1_sys = set(['C3', 'C8', 'G1.C1', 'G2.C5', 'indep1', 'G1', 'G2', ''])
+        indep1_ins = {'C8.b', 'G2.C5.a', 'G1.C1.a'}
+        indep1_outs = {'C8.y', 'G1.C1.z', 'G2.C5.x', 'indep1.x'}
+        indep1_sys = {'C8', 'G1.C1', 'G2.C5', 'indep1', 'G1', 'G2', ''}
 
-        dct, systems = relevant['C8.y']['indep1.x']
-        inputs = dct['input']
-        outputs = dct['output']
+        inputs, outputs, systems = relevant._all_relevant('indep1.x', 'C8.y')
 
         self.assertEqual(inputs, indep1_ins)
         self.assertEqual(outputs, indep1_outs)
         self.assertEqual(systems, indep1_sys)
 
-        dct, systems = relevant['C8.y']['indep1.x']
-        inputs = dct['input']
-        outputs = dct['output']
+        self.assertTrue('indep2.x' not in outputs)
+
+        inputs, outputs, systems = relevant._all_relevant(['indep1.x', 'indep2.x'], 'C8.y')
 
         self.assertEqual(inputs, indep1_ins)
         self.assertEqual(outputs, indep1_outs)
         self.assertEqual(systems, indep1_sys)
 
-        indep2_ins = set(['C8.a', 'G2.C6.a', 'G2.C7.b'])
-        indep2_outs = set(['C8.y', 'G2.C6.y', 'G2.C7.x', 'indep2.x'])
-        indep2_sys = set(['C8', 'G2.C6', 'G2.C7', 'indep2', 'G2', ''])
-
-        dct, systems = relevant['C8.y']['indep2.x']
-        inputs = dct['input']
-        outputs = dct['output']
-
-        self.assertEqual(inputs, indep2_ins)
-        self.assertEqual(outputs, indep2_outs)
-        self.assertEqual(systems, indep2_sys)
-
-        dct, systems = relevant['C8.y']['indep2.x']
-        inputs = dct['input']
-        outputs = dct['output']
-
-        self.assertEqual(inputs, indep2_ins)
-        self.assertEqual(outputs, indep2_outs)
-        self.assertEqual(systems, indep2_sys)
-
-        dct, systems = relevant['C8.y']['@all']
-        inputs = dct['input']
-        outputs = dct['output']
-
-        self.assertEqual(inputs, indep1_ins | indep2_ins)
-        self.assertEqual(outputs, indep1_outs | indep2_outs)
-        self.assertEqual(systems, indep1_sys | indep2_sys)
-
     def test_system_setup_and_configure(self):
         # Test that we can change solver settings on a subsystem in a system's setup method.
         # Also assures that highest system's settings take precedence.
@@ -1561,13 +1504,13 @@ class TestProblem(unittest.TestCase):
             p.setup()
             p.run_model()
 
-            inputs = p.model.list_inputs(out_stream=None)
+            inputs = p.model.list_inputs(out_stream=None, prom_name=False)
             self.assertEqual(sorted(inputs), [
                 ('comp3.a', {'val': [2.]}),
                 ('comp3.b', {'val': [3.]})
             ], "Inputs don't match when added in %s." % where)
 
-            outputs = p.model.list_outputs(out_stream=None)
+            outputs = p.model.list_outputs(out_stream=None, prom_name=False)
             self.assertEqual(sorted(outputs), [
                 ('comp1.a',   {'val': [2.]}),
                 ('comp1.foo', {'val': [1.]}),
@@ -1617,13 +1560,13 @@ class TestProblem(unittest.TestCase):
         p.setup()
         p.run_model()
 
-        inputs = p.model.list_inputs(out_stream=None)
+        inputs = p.model.list_inputs(out_stream=None, prom_name=False)
         self.assertEqual(sorted(inputs), [
             ('mcomp.a',     {'val': [2.]}),
             ('sub.mcomp.a', {'val': [2.]}),
         ])
 
-        outputs = p.model.list_outputs(out_stream=None)
+        outputs = p.model.list_outputs(out_stream=None, prom_name=False)
         self.assertEqual(sorted(outputs), [
             ('indep.a',      {'val': [2.]}),
             ('mcomp.a2',     {'val': [4.]}),
@@ -1635,7 +1578,7 @@ class TestProblem(unittest.TestCase):
         p.setup()
         p.run_model()
 
-        inputs = p.model.list_inputs(out_stream=None)
+        inputs = p.model.list_inputs(out_stream=None, prom_name=False)
         self.assertEqual(sorted(inputs), [
             ('mcomp.a',     {'val': [2.]}),
             ('mcomp.b',     {'val': [3.]}),
@@ -1643,7 +1586,7 @@ class TestProblem(unittest.TestCase):
             ('sub.mcomp.b', {'val': [3.]}),
         ])
 
-        outputs= p.model.list_outputs(out_stream=None)
+        outputs= p.model.list_outputs(out_stream=None, prom_name=False)
         self.assertEqual(sorted(outputs), [
             ('indep.a',      {'val': [2.]}),
             ('indep.b',      {'val': [3.]}),
@@ -1724,7 +1667,7 @@ class TestProblem(unittest.TestCase):
             hooks._unregister_hook('final_setup', class_name='Problem')
             hooks.use_hooks = False
 
-    def test_list_problem_vars(self):
+    def test_list_driver_vars(self):
         model = SellarDerivatives()
         model.nonlinear_solver = om.NonlinearBlockGS()
 
@@ -1751,11 +1694,11 @@ class TestProblem(unittest.TestCase):
         finally:
             sys.stdout = stdout
         output = strout.getvalue().split('\n')
-        self.assertEquals(output[1], r'Design Variables')
+        self.assertEqual(output[1], r'Design Variables')
         self.assertRegex(output[5], r'^z +\|[0-9. e+-]+\| +2')
-        self.assertEquals(output[9], r'Constraints')
+        self.assertEqual(output[9], r'Constraints')
         self.assertRegex(output[14], r'^con2 +\[[0-9. e+-]+\] +1')
-        self.assertEquals(output[17], r'Objectives')
+        self.assertEqual(output[17], r'Objectives')
         self.assertRegex(output[21], r'^obj +\[[0-9. e+-]+\] +1')
 
         # With show_promoted_name=False
@@ -1763,20 +1706,20 @@ class TestProblem(unittest.TestCase):
         strout = StringIO()
         sys.stdout = strout
         try:
-            prob.list_problem_vars(show_promoted_name=False)
+            prob.list_driver_vars(show_promoted_name=False)
         finally:
             sys.stdout = stdout
         output = strout.getvalue().split('\n')
         self.assertRegex(output[5], r'^z +\|[0-9. e+-]+\| +2')
-        self.assertRegex(output[14], r'^con_cmp2.con2 +\[[0-9. e+-]+\] +1')
-        self.assertRegex(output[21], r'^obj_cmp.obj +\[[0-9. e+-]+\] +1')
+        self.assertRegex(output[14], r'^con2 +\[[0-9. e+-]+\] +1')
+        self.assertRegex(output[21], r'^obj +\[[0-9. e+-]+\] +1')
 
         # With all the optional columns
         stdout = sys.stdout
         strout = StringIO()
         sys.stdout = strout
         try:
-            prob.list_problem_vars(
+            prob.list_driver_vars(
                 desvar_opts=['lower', 'upper', 'ref', 'ref0',
                              'indices', 'adder', 'scaler',
                              'parallel_deriv_color',
@@ -1806,19 +1749,19 @@ class TestProblem(unittest.TestCase):
         strout = StringIO()
         sys.stdout = strout
         try:
-            prob.list_problem_vars(print_arrays=True,
-                                   desvar_opts=['lower', 'upper', 'ref', 'ref0',
-                                                'indices', 'adder', 'scaler',
-                                                'parallel_deriv_color',
-                                                'cache_linear_solution'],
-                                   cons_opts=['lower', 'upper', 'equals', 'ref', 'ref0',
-                                              'indices', 'adder', 'scaler', 'linear',
-                                              'parallel_deriv_color',
-                                              'cache_linear_solution'],
-                                   objs_opts=['ref', 'ref0',
-                                              'indices', 'adder', 'scaler',
-                                              'parallel_deriv_color',
-                                              'cache_linear_solution'],
+            l = prob.list_driver_vars(print_arrays=True,
+                                       desvar_opts=['lower', 'upper', 'ref', 'ref0',
+                                                    'indices', 'adder', 'scaler',
+                                                    'parallel_deriv_color',
+                                                    'cache_linear_solution'],
+                                       cons_opts=['lower', 'upper', 'equals', 'ref', 'ref0',
+                                                  'indices', 'adder', 'scaler', 'linear',
+                                                  'parallel_deriv_color',
+                                                  'cache_linear_solution'],
+                                        objs_opts=['ref', 'ref0',
+                                                   'indices', 'adder', 'scaler',
+                                                   'parallel_deriv_color',
+                                                   'cache_linear_solution'],
                                    )
         finally:
             sys.stdout = stdout
@@ -1830,31 +1773,68 @@ class TestProblem(unittest.TestCase):
         self.assertRegex(output[12], r'^\s+upper:')
         self.assertRegex(output[13], r'^\s+array+\(+\[[0-9., e+-]+\]+\)')
 
+        # design vars
+        self.assertEqual(l['design_vars'][0][1]['name'], 'z')
+        self.assertEqual(l['design_vars'][0][1]['size'], 2)
+        assert(all(l['design_vars'][0][1]['val'] == prob.get_val('z')))
+        self.assertEqual(l['design_vars'][0][1]['scaler'], None)
+        self.assertEqual(l['design_vars'][0][1]['adder'], None)
+
+        self.assertEqual(l['design_vars'][1][1]['name'], 'x')
+        self.assertEqual(l['design_vars'][1][1]['size'], 1)
+        assert(all(l['design_vars'][1][1]['val'] == prob.get_val('x')))
+        self.assertEqual(l['design_vars'][1][1]['scaler'], None)
+        self.assertEqual(l['design_vars'][1][1]['adder'], None)
+
+        # constraints
+        self.assertEqual(l['constraints'][0][1]['name'], 'con1')
+        self.assertEqual(l['constraints'][0][1]['size'], 1)
+        assert(all(l['constraints'][0][1]['val'] == prob.get_val('con1')))
+        self.assertEqual(l['constraints'][0][1]['scaler'], None)
+        self.assertEqual(l['constraints'][0][1]['adder'], None)
+
+        self.assertEqual(l['constraints'][1][1]['name'], 'con2')
+        self.assertEqual(l['constraints'][1][1]['size'], 1)
+        assert(all(l['constraints'][1][1]['val'] == prob.get_val('con2')))
+        self.assertEqual(l['constraints'][1][1]['scaler'], None)
+        self.assertEqual(l['constraints'][1][1]['adder'], None)
+
+        # objectives
+        self.assertEqual(l['objectives'][0][1]['name'], 'obj')
+        self.assertEqual(l['objectives'][0][1]['size'], 1)
+        assert(all(l['objectives'][0][1]['val'] == prob.get_val('obj')))
+        self.assertEqual(l['objectives'][0][1]['scaler'], None)
+        self.assertEqual(l['objectives'][0][1]['adder'], None)
+
     def test_list_problem_vars_before_final_setup(self):
         prob = om.Problem()
         prob.model.add_subsystem('parab', Paraboloid(), promotes_inputs=['x', 'y'])
         prob.model.add_subsystem('const', om.ExecComp('g = x + y'), promotes_inputs=['x', 'y'])
         prob.model.set_input_defaults('x', 3.0)
         prob.model.set_input_defaults('y', -4.0)
+
         prob.driver = om.ScipyOptimizeDriver()
         prob.driver.options['optimizer'] = 'COBYLA'
+
         prob.model.add_design_var('x', lower=-50, upper=50)
         prob.model.add_design_var('y', lower=-50, upper=50)
         prob.model.add_objective('parab.f_xy')
         prob.model.add_constraint('const.g', lower=0, upper=10.)
+
         prob.setup()
 
-        msg = "Problem .*: Problem.list_problem_vars\(\) cannot be called before " \
-                         "`Problem\.run_model\(\)`, `Problem\.run_driver\(\)`, or " \
-                         "`Problem\.final_setup\(\)`\."
-        with self.assertRaisesRegex(RuntimeError, msg):
-            prob.list_problem_vars()
+        msg = f"Problem {prob._get_inst_id()}: Problem.list_driver_vars() cannot be called " \
+              "before `Problem.run_model()`, `Problem.run_driver()`, or `Problem.final_setup()`."
+
+        with self.assertRaises(RuntimeError) as err:
+            prob.list_driver_vars()
+        self.assertEqual(str(err.exception), msg)
 
     def test_list_problem_w_multi_constraints(self):
         p = om.Problem()
 
-        exec = om.ExecComp(['y = x**2',
-                            'z = a + x**2'],
+        exec = om.ExecComp(['y = a + x**2',
+                            'z = a * x**2'],
                         a={'shape': (1,)},
                         y={'shape': (101,)},
                         x={'shape': (101,)},
@@ -1880,7 +1860,7 @@ class TestProblem(unittest.TestCase):
         strout = StringIO()
         sys.stdout = strout
         try:
-            p.list_problem_vars()
+            p.list_driver_vars()
         finally:
             sys.stdout = stdout
 
@@ -1907,17 +1887,17 @@ class TestProblem(unittest.TestCase):
         model.add_constraint('f', indices=[1], flat_indices=True, lower=10.0)
         model.add_constraint('f', indices=[5], flat_indices=True, alias='g', lower=0.5)
 
-        msg = "Constraint alias 'f' is a duplicate of an existing alias or variable name."
+        msg = "<class Group>: Constraint alias 'f' is a duplicate of an existing alias or variable name."
         with self.assertRaises(Exception) as cm:
             model.add_constraint('f', indices=[3], flat_indices=True, alias='f', lower=0.5)
 
         self.assertEqual(str(cm.exception), msg)
 
-        msg = "\nCollected errors for problem 'constraint_alias_duplicate_errors':" + \
-              "\n   <model> <class Group>: Constraint alias 'g' on 'comp1.f' is the same name as " + \
+        msg = "<model> <class Group>: Constraint alias 'g' on 'comp1.f' is the same name as " + \
               "an existing variable."
+        prob.setup()
         with self.assertRaises(Exception) as cm:
-            prob.setup()
+            prob.final_setup()
 
         self.assertEqual(str(cm.exception), msg)
 
@@ -1980,7 +1960,7 @@ class TestProblem(unittest.TestCase):
         sys.stdout = strout
 
         try:
-            prob.list_problem_vars()
+            prob.list_driver_vars()
         finally:
             sys.stdout = stdout
         output = strout.getvalue().split('\n')
@@ -1995,7 +1975,7 @@ class TestProblem(unittest.TestCase):
         sys.stdout = strout
 
         try:
-            prob.list_problem_vars(driver_scaling=False)
+            prob.list_driver_vars(driver_scaling=False)
         finally:
             sys.stdout = stdout
         output = strout.getvalue().split('\n')
@@ -2004,7 +1984,7 @@ class TestProblem(unittest.TestCase):
         self.assertTrue('-20.' in output[14]) # con
         self.assertTrue('3.18' in output[21]) # obj
 
-    def test_feature_list_problem_vars(self):
+    def test_feature_list_driver_vars(self):
 
         prob = om.Problem(model=SellarDerivatives())
         model = prob.model
@@ -2023,7 +2003,7 @@ class TestProblem(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
-        prob.list_problem_vars(print_arrays=True,
+        prob.list_driver_vars(print_arrays=True,
                                desvar_opts=['lower', 'upper', 'ref', 'ref0',
                                             'indices', 'adder', 'scaler',
                                             'parallel_deriv_color'],
@@ -2058,12 +2038,12 @@ class TestProblem(unittest.TestCase):
                                        promotes_inputs=['x'])
 
         c1.add_design_var('x', lower=0, upper=5)
+        prob.setup()
 
         with self.assertRaises(Exception) as cm:
-            prob.setup()
+            prob.final_setup()
 
-        msg = "\nCollected errors for problem 'output_as_input_err':" + \
-              "\n   <model> <class Group>: Design variable 'x' is connected to 'initial_comp.x', " + \
+        msg = "<model> <class Group>: Design variable 'x' is connected to 'initial_comp.x', " + \
               "but 'initial_comp.x' is not an IndepVarComp or ImplicitComp output."
         self.assertEqual(str(cm.exception), msg)
 
@@ -2108,6 +2088,7 @@ class TestProblem(unittest.TestCase):
             self.fail("'setup raised RuntimeError unexpectedly")
 
 
+@use_tempdirs
 class RelevanceTestCase(unittest.TestCase):
     def _setup_relevance_problem(self):
         p = om.Problem()
@@ -2122,8 +2103,8 @@ class RelevanceTestCase(unittest.TestCase):
         model.add_subsystem('C2', MultComp(3.))
         model.add_subsystem('C3', MultComp(5.))
         model.add_subsystem('C4', MultComp(7.))
-        model.add_subsystem('C5', MultComp(9.))
-        model.add_subsystem('C6', MultComp(11.))
+        model.add_subsystem('C5', MultComp(.1))
+        model.add_subsystem('C6', MultComp(.01))
 
         model.connect('indeps.a', 'C1.x')
         model.connect('indeps.b', ['C1.y', 'C2.x'])
@@ -2140,9 +2121,11 @@ class RelevanceTestCase(unittest.TestCase):
         p = self._setup_relevance_problem()
         p.model.connect('C5.fxy', 'C4.y')
         p.model.connect('C6.fxy', 'C5.y')
+        p.model.nonlinear_solver = om.NonlinearBlockGS(maxiter=500)
+        p.model.linear_solver = om.LinearBlockGS(maxiter=500)
         return p
 
-    def _finish_setup_and_check(self, p, expected):
+    def _finish_setup_and_check(self, p, expected, approx=False):
         p.setup()
 
         p['indeps.a'] = 2.
@@ -2154,16 +2137,26 @@ class RelevanceTestCase(unittest.TestCase):
 
         p.run_model()
 
-        p.run_driver()
-
         allcomps = [getattr(p.model, f"C{i}") for i in range(1, 7)]
+
+        if approx:
+            for c in allcomps:
+                c._reset_counts(names=['_compute_wrapper'])
+            p.compute_totals()
+        else:
+            p.run_driver()
+
         ran_linearize = [c.name for c in allcomps if c._counts['_linearize'] > 0]
         ran_compute_partials = [c.name for c in allcomps if c._counts['_compute_partials_wrapper'] > 0]
         ran_solve_linear = [c.name for c in allcomps if c._counts['_solve_linear'] > 0]
 
-        self.assertEqual(ran_linearize, expected)
-        self.assertEqual(ran_compute_partials, expected)
-        self.assertEqual(ran_solve_linear, expected)
+        if approx:
+            for c in allcomps:
+                self.assertEqual(c._counts['_compute_wrapper'], expected[c.name], f"for {c.name}")
+        else:
+            self.assertEqual(ran_linearize, expected)
+            self.assertEqual(ran_compute_partials, expected)
+            self.assertEqual(ran_solve_linear, expected)
 
     def test_relevance(self):
         p = self._setup_relevance_problem()
@@ -2175,6 +2168,17 @@ class RelevanceTestCase(unittest.TestCase):
 
         self._finish_setup_and_check(p, ['C2', 'C4', 'C6'])
 
+    def test_relevance_approx(self):
+        p = self._setup_relevance_problem()
+
+        p.driver = om.ScipyOptimizeDriver(disp=False, tol=1e-9, optimizer='SLSQP')
+        p.model.add_design_var('indeps.b', lower=-50., upper=50.)
+        p.model.add_objective('C6.fxy')
+        p.model.add_constraint('C4.fxy', upper=1000.)
+        p.model.approx_totals(method='cs')
+
+        self._finish_setup_and_check(p, {'C2': 1, 'C4': 1, 'C6': 1, 'C1': 0, 'C3': 0, 'C5': 0}, approx=True)
+
     def test_relevance2(self):
         p = self._setup_relevance_problem()
 
@@ -2215,6 +2219,78 @@ class RelevanceTestCase(unittest.TestCase):
 
         self._finish_setup_and_check(p, ['C2', 'C3', 'C4', 'C5', 'C6'])
 
+    def test_list_indep_vars(self):
+        prob = om.Problem()
+        prob.model = SellarDerivatives()
+        prob.model.add_design_var('x')
+        prob.model.add_design_var('z')
+
+        prob.setup()
+        prob.final_setup()
+
+        strout = StringIO()
+        all_indep_var_names = [name for name, _ in prob.list_indep_vars(out_stream=strout)]
+
+        self.assertIn('x', all_indep_var_names)
+        self.assertIn('z', all_indep_var_names)
+
+        output = strout.getvalue()
+        self.assertRegex(output.split('\n')[1], r'Problem \w+ Independent Variables')
+        self.assertEqual(output.split('\n')[3].split(), ['name', 'units', 'val'])
+        self.assertRegex(output.split('\n')[5], r'\s*z\s+None\s+|[0-9.]+|')
+        self.assertRegex(output.split('\n')[6], r'\s*x\s+None\s+|[0-9.]+|')
+
+        strout = StringIO()
+        indep_var_names_no_desvars = [name for name, _ in
+                                      prob.list_indep_vars(include_design_vars=False,
+                                                           out_stream=strout)]
+
+        self.assertNotIn('x', indep_var_names_no_desvars)
+        self.assertNotIn('z', indep_var_names_no_desvars)
+
+        output = strout.getvalue()
+        self.assertRegex(output.split('\n')[1], r'Problem \w+ Independent Variables')
+        self.assertEqual(output.split('\n')[3].split(), ['None', 'found'])
+
+    def test_list_indep_vars_duplicate_inputs(self):
+        prob = om.Problem()
+        prob.model.add_subsystem('a', om.ExecComp('y = x**2'),
+                                 promotes_inputs=['x'], promotes_outputs=['y'])
+
+        prob.model.add_subsystem('b', om.ExecComp('z = x**3'),
+                                 promotes_inputs=['x'], promotes_outputs=['z'])
+
+        prob.model.add_design_var('x', lower=-1, upper=1)
+        prob.model.add_objective('y')
+
+        prob.setup()
+
+        prob.final_setup()
+
+        indep_vars = prob.list_indep_vars()
+
+        self.assertEqual(len(indep_vars), 1)
+
+    def test_list_indep_vars_no_auto_ivc(self):
+        prob = om.Problem()
+
+        prob.model.add_subsystem('indep_theta', om.IndepVarComp('theta'))
+        prob.model.add_subsystem('indep_r', om.IndepVarComp('r'))
+
+        prob.model.add_subsystem('xComp', om.ExecComp('x = r1*cos(theta1)'))
+        prob.model.add_subsystem('yComp', om.ExecComp('y = r2*sin(theta2)'))
+
+        prob.model.connect('indep_theta.theta', ['xComp.theta1', 'yComp.theta2'])
+        prob.model.connect('indep_r.r', ['xComp.r1', 'yComp.r2'])
+
+        prob.setup()
+        prob.final_setup()
+
+        indep_vars = prob.list_indep_vars()
+        self.assertEqual(indep_vars[0][0], 'indep_r.r')
+        self.assertEqual(indep_vars[1][0], 'indep_theta.theta')
+
+
 class NestedProblemTestCase(unittest.TestCase):
 
     def test_nested_prob(self):
@@ -2300,8 +2376,7 @@ class NestedProblemTestCase(unittest.TestCase):
         prob.run_model()
 
         totals = prob.check_totals(of='f_xy', wrt=['x', 'y'], method='cs', out_stream=None)
-        for key, val in totals.items():
-            assert_near_equal(val['rel error'][0], 0.0, 1e-12)
+        assert_check_totals(totals)
 
     def test_nested_prob_default_naming(self):
         import openmdao.core.problem
