@@ -1,6 +1,7 @@
 """
 Test DOE Driver and Generators.
 """
+from io import StringIO
 import unittest
 
 import os
@@ -16,8 +17,9 @@ from openmdao.test_suite.components.paraboloid import Paraboloid
 from openmdao.test_suite.components.paraboloid_distributed import DistParab
 from openmdao.test_suite.groups.parallel_groups import FanInGrouped
 
-from openmdao.utils.assert_utils import assert_near_equal
+from openmdao.utils.assert_utils import assert_near_equal, assert_warning
 from openmdao.utils.general_utils import run_driver, printoptions
+from openmdao.utils.om_warnings import OMDeprecationWarning
 from openmdao.utils.testing_utils import use_tempdirs
 
 from openmdao.utils.mpi import MPI
@@ -28,9 +30,9 @@ except ImportError:
     PETScVector = None
 
 try:
-    import pyDOE2
+    import pyDOE3
 except ImportError:
-    pyDOE2 = None
+    pyDOE3 = None
 
 
 class ParaboloidArray(om.ExplicitComponent):
@@ -100,27 +102,27 @@ class TestErrors(unittest.TestCase):
                          "DOEDriver requires an instance of DOEGenerator, "
                          "but an instance of Problem was found.")
 
-    @unittest.skipIf(pyDOE2, "only runs if 'pyDOE2' is not installed")
-    def test_no_pyDOE2(self):
+    @unittest.skipIf(pyDOE3, "only runs if 'pyDOE3' is not installed")
+    def test_no_pyDOE3(self):
         with self.assertRaises(RuntimeError) as err:
             om.FullFactorialGenerator(levels=3)
 
         self.assertEqual(str(err.exception),
-                         "FullFactorialGenerator requires the 'pyDOE2' package, "
+                         "FullFactorialGenerator requires the 'pyDOE3' package, "
                          "which can be installed with one of the following commands:\n"
                          "    pip install openmdao[doe]\n"
-                         "    pip install pyDOE2")
+                         "    pip install pyDOE3")
 
         with self.assertRaises(RuntimeError) as err:
             om.DOEDriver(generator=om.FullFactorialGenerator(levels=3))
 
         self.assertEqual(str(err.exception),
-                         "FullFactorialGenerator requires the 'pyDOE2' package, "
+                         "FullFactorialGenerator requires the 'pyDOE3' package, "
                          "which can be installed with one of the following commands:\n"
                          "    pip install openmdao[doe]\n"
-                         "    pip install pyDOE2")
+                         "    pip install pyDOE3")
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_lhc_criterion(self):
         with self.assertRaises(ValueError) as err:
             om.LatinHypercubeGenerator(criterion='foo')
@@ -267,8 +269,8 @@ class TestDOEDriver(unittest.TestCase):
 
         # data contains a list of list, but one has the wrong length
         cases = [
-            [['p1.x', 0.], ['p2.y', 0.]],
-            [['p1.x', 1.], ['p2.y', 1., 'foo']]
+            [['x', 0.], ['y', 0.]],
+            [['x', 1.], ['y', 1., 'foo']]
         ]
 
         prob.driver = om.DOEDriver(generator=om.ListGenerator(cases))
@@ -277,12 +279,12 @@ class TestDOEDriver(unittest.TestCase):
             prob.run_driver()
         self.assertEqual(str(err.exception), "Invalid DOE case found, "
                          "expecting a list of name/value pairs:\n"
-                         "[['p1.x', 1.0], ['p2.y', 1.0, 'foo']]")
+                         "[['x', 1.0], ['y', 1.0, 'foo']]")
 
         # data contains a list of list, but one case has an invalid design var
         cases = [
-            [['p1.x', 0.], ['p2.y', 0.]],
-            [['p1.x', 1.], ['p2.z', 1.]]
+            [['x', 0.], ['y', 0.]],
+            [['x', 1.], ['z', 1.]]
         ]
 
         prob.driver = om.DOEDriver(generator=om.ListGenerator(cases))
@@ -290,12 +292,12 @@ class TestDOEDriver(unittest.TestCase):
         with self.assertRaises(RuntimeError) as err:
             prob.run_driver()
         self.assertEqual(str(err.exception), "Invalid DOE case found, "
-                         "'p2.z' is not a valid design variable:\n"
-                         "[['p1.x', 1.0], ['p2.z', 1.0]]")
+                         "'z' is not a valid design variable:\n"
+                         "[['x', 1.0], ['z', 1.0]]")
 
         # data contains a list of list, but one case has multiple invalid design vars
         cases = [
-            [['p1.x', 0.], ['p2.y', 0.]],
+            [['x', 0.], ['y', 0.]],
             [['p1.y', 1.], ['p2.z', 1.]]
         ]
 
@@ -348,7 +350,7 @@ class TestDOEDriver(unittest.TestCase):
             for name in ('x', 'y', 'f_xy'):
                 self.assertEqual(outputs[name], expected_case[name])
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_csv_array(self):
         prob = om.Problem()
         model = prob.model
@@ -484,18 +486,11 @@ class TestDOEDriver(unittest.TestCase):
             for case in cases:
                 writer.writerow([np.ones((2, 2)) * val for _, val in case])
 
-        from packaging.version import Version
-        if Version(np.__version__) >= Version("1.14"):
-            opts = {'legacy': '1.13'}
-        else:
-            opts = {}
-
-        with printoptions(**opts):
-            # have to use regex to handle differences in numpy print formats for shape
-            msg = f"Error assigning p1.x = \[ 0.  0.  0.  0.\]: could not broadcast " \
-                  f"input array from shape \(4.*\) into shape \(1.*\)"
-            with self.assertRaisesRegex(ValueError, msg):
-                prob.run_driver()
+        with self.assertRaises(ValueError) as err:
+            prob.run_driver()
+        self.assertEqual(str(err.exception),
+                         "Error assigning x = [0. 0. 0. 0.]: could not broadcast "
+                         "input array from shape (4,) into shape (1,)")
 
     def test_uniform(self):
         prob = om.Problem()
@@ -535,7 +530,7 @@ class TestDOEDriver(unittest.TestCase):
             for name in ('x', 'y'):
                 assert_near_equal(outputs[name], expected_case[name], 1e-4)
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_full_factorial(self):
         prob = om.Problem()
         model = prob.model
@@ -566,7 +561,7 @@ class TestDOEDriver(unittest.TestCase):
             for name in ('x', 'y', 'f_xy'):
                 self.assertEqual(outputs[name], expected_case[name])
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_full_factorial_factoring(self):
 
         class Digits2Num(om.ExplicitComponent):
@@ -606,14 +601,14 @@ class TestDOEDriver(unittest.TestCase):
         cr = om.CaseReader("cases.sql")
         cases = cr.list_cases('driver', out_stream=None)
 
-        objs = [int(cr.get_case(case).outputs['f']) for case in cases]
+        objs = [cr.get_case(case).outputs['f'].item() for case in cases]
 
         self.assertEqual(len(objs), 16)
         # Testing uniqueness. If all elements are unique, it should be the same length as the
         # number of cases
         self.assertEqual(len(set(objs)), 16)
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_full_factorial_array(self):
         prob = om.Problem()
         model = prob.model
@@ -655,7 +650,7 @@ class TestDOEDriver(unittest.TestCase):
             self.assertEqual(outputs['xy'][0], expected_case['xy'][0])
             self.assertEqual(outputs['xy'][1], expected_case['xy'][1])
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_full_fact_dict_levels(self):
         # Specifying levels only for one DV, the other is defaulted
         prob = om.Problem()
@@ -697,7 +692,7 @@ class TestDOEDriver(unittest.TestCase):
             self.assertEqual(outputs['y'], expected_case['y'])
             self.assertEqual(outputs['f_xy'], expected_case['f_xy'])
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_generalized_subset(self):
         # All DVs have the same number of levels
         prob = om.Problem()
@@ -733,7 +728,7 @@ class TestDOEDriver(unittest.TestCase):
             for name in ('x', 'y', 'f_xy'):
                 self.assertEqual(outputs[name], expected_case[name])
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_generalized_subset_dict_levels(self):
         # Number of variables specified individually for all DVs (scalars).
         prob = om.Problem()
@@ -776,7 +771,7 @@ class TestDOEDriver(unittest.TestCase):
             for name in ('x', 'y', 'f_xy'):
                 self.assertAlmostEqual(outputs[name][0], expected_case[name][0])
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_generalized_subset_array(self):
         # Number of levels specified individually for all DVs (arrays).
 
@@ -817,13 +812,13 @@ class TestDOEDriver(unittest.TestCase):
         cr = om.CaseReader("cases.sql")
         cases = cr.list_cases('driver', out_stream=None)
 
-        objs = [int(cr.get_case(case).outputs['f']) for case in cases]
+        objs = [cr.get_case(case).outputs['f'].item() for case in cases]
 
-        self.assertEqual(len(objs), 104)  # The number can be verified with standalone pyDOE2
+        self.assertEqual(len(objs), 104)  # The number can be verified with standalone pyDOE3
         # Testing uniqueness. If all elements are unique, it should be the same length as the number of cases
         self.assertEqual(len(set(objs)), 104)
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_plackett_burman(self):
         prob = om.Problem()
         model = prob.model
@@ -860,7 +855,7 @@ class TestDOEDriver(unittest.TestCase):
             for name in ('x', 'y', 'f_xy'):
                 self.assertEqual(outputs[name], expected_case[name])
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_box_behnken(self):
         upper = 10.
         center = 1
@@ -921,7 +916,7 @@ class TestDOEDriver(unittest.TestCase):
             for name in ('x', 'y', 'z'):
                 self.assertEqual(outputs[name], expected_case[name])
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_latin_hypercube(self):
         samples = 4
 
@@ -979,8 +974,8 @@ class TestDOEDriver(unittest.TestCase):
 
         for case, expected_case in zip(cases, expected):
             outputs = cr.get_case(case).outputs
-            x = outputs['x']
-            y = outputs['y']
+            x = outputs['x'].item()
+            y = outputs['y'].item()
 
             bucket = int((x + x_offset) / (x_bucket_size / samples))
             x_buckets_filled.add(bucket)
@@ -994,7 +989,7 @@ class TestDOEDriver(unittest.TestCase):
         self.assertEqual(x_buckets_filled, all_buckets)
         self.assertEqual(y_buckets_filled, all_buckets)
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_latin_hypercube_array(self):
         samples = 4
 
@@ -1063,7 +1058,7 @@ class TestDOEDriver(unittest.TestCase):
         self.assertEqual(x_buckets_filled, all_buckets)
         self.assertEqual(y_buckets_filled, all_buckets)
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_latin_hypercube_center(self):
         samples = 4
         upper = 10.
@@ -1110,8 +1105,8 @@ class TestDOEDriver(unittest.TestCase):
 
         for case in cases:
             outputs = cr.get_case(case).outputs
-            x = float(outputs['indep.x'])
-            y = float(outputs['indep.y'])
+            x = outputs['indep.x'].item()
+            y = outputs['indep.y'].item()
 
             x_buckets_filled.add(int(x/bucket_size))
             y_buckets_filled.add(int(y/bucket_size))
@@ -1122,7 +1117,7 @@ class TestDOEDriver(unittest.TestCase):
         self.assertEqual(x_buckets_filled, all_buckets)
         self.assertEqual(y_buckets_filled, all_buckets)
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_record_bug(self):
         # There was a bug that caused values to be recorded in driver_scaled form.
 
@@ -1382,7 +1377,7 @@ class TestDOEDriver(unittest.TestCase):
                 self.assertEqual(outputs[name], expected_case[name])
                 self.assertTrue(isinstance(outputs[name], int))
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_desvar_indices(self):
         prob = om.Problem()
         prob.model.add_subsystem('comp', om.ExecComp('y=x**2',
@@ -1438,7 +1433,7 @@ class TestDOEDriver(unittest.TestCase):
         for name in ('x', 'y', 'z'):
             assert_near_equal(outputs[name], prob[name])
 
-    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
     def test_multi_constraint_doe(self):
         prob = om.Problem()
         prob.model.add_subsystem('comp', om.ExecComp('y=x**2 + b',
@@ -1500,6 +1495,43 @@ class TestDOEDriver(unittest.TestCase):
             for dv in ('x', 'y'):
                 self.assertEqual(derivs['f_xy', dv], expected_deriv['f_xy', dv])
 
+    @unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
+    def test_derivative_scaled_recording(self):
+        prob = om.Problem()
+        model = prob.model
+
+        model.add_subsystem('comp', Paraboloid(), promotes=['x', 'y', 'f_xy'])
+        model.set_input_defaults('x', 0.0)
+        model.set_input_defaults('y', 0.0)
+        model.add_design_var('x', lower=0.0, upper=1.0, ref=5.0)
+        model.add_design_var('y', lower=0.0, upper=1.0, ref=10.0)
+        model.add_objective('f_xy', ref=2.0)
+
+        prob.driver = om.DOEDriver(generator=om.FullFactorialGenerator(levels=3))
+        prob.driver.add_recorder(om.SqliteRecorder("cases.sql"))
+        prob.driver.recording_options['record_derivatives'] = True
+
+        prob.setup()
+        prob.run_driver()
+        prob.cleanup()
+
+        expected_vals = self.expected_fullfact3
+        expected_derivs = self.expected_fullfact3_derivs
+
+        cr = om.CaseReader("cases.sql")
+        cases = cr.list_cases('driver', out_stream=None)
+
+        self.assertEqual(len(cases), 9)
+
+        for case, expected_val, expected_deriv in zip(cases, expected_vals, expected_derivs):
+            outputs = cr.get_case(case).outputs
+            for name in ('x', 'y', 'f_xy'):
+                self.assertEqual(outputs[name], expected_val[name])
+
+            derivs = cr.get_case(case).derivatives
+            for dv in ('x', 'y'):
+                self.assertEqual(derivs['f_xy', dv], expected_deriv['f_xy', dv])
+
     def test_derivative_no_recording(self):
         prob = om.Problem()
         model = prob.model
@@ -1532,7 +1564,7 @@ class TestDOEDriver(unittest.TestCase):
 @use_tempdirs
 class TestDOEDriverListVars(unittest.TestCase):
 
-    def test_list_problem_vars(self):
+    def test_list_driver_vars(self):
         # this passes if no exception is raised
 
         prob = om.Problem()
@@ -1563,13 +1595,13 @@ class TestDOEDriverListVars(unittest.TestCase):
         prob.run_driver()
         prob.cleanup()
 
-        prob.list_problem_vars()
+        prob.list_driver_vars()
 
 
 @use_tempdirs
 class TestDOEDriverListVars(unittest.TestCase):
 
-    def test_list_problem_vars(self):
+    def test_list_driver_vars(self):
         # this passes if no exception is raised
 
         prob = om.Problem()
@@ -1600,7 +1632,21 @@ class TestDOEDriverListVars(unittest.TestCase):
         prob.run_driver()
         prob.cleanup()
 
-        prob.list_problem_vars()
+        f = StringIO()
+        prob.list_driver_vars(out_stream=f)
+        output = f.getvalue()
+
+        self.assertIn('x     -1   0', output)
+        self.assertIn('y     3    0', output)
+        self.assertIn('f_xy  59   0', output)
+
+        expected_warning = 'Method `list_problem_vars` has been ' \
+                           'renamed `list_driver_vars`.\nPlease update ' \
+                           'your code to use list_driver_vars to avoid ' \
+                           'this warning.'
+
+        with assert_warning(OMDeprecationWarning, expected_warning):
+            prob.list_problem_vars()
 
 
 @unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
@@ -2155,7 +2201,7 @@ class TestParallelDOE2proc(unittest.TestCase):
 
 
 @unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
-@unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
+@unittest.skipUnless(pyDOE3, "requires 'pyDOE3', pip install openmdao[doe]")
 @use_tempdirs
 class TestParallelDistribDOE(unittest.TestCase):
 
